@@ -90,6 +90,7 @@ class CardsManager:
 
 class ConfigManager:
     def __init__(self):
+        self.deck_generator = json.loads(pkg_resources.read_text(json_files, 'generated_decks.json'))
         json_file = pkg_resources.read_text(assets, 'game_config.json')
         self.game_config = json.loads(json_file)
         all_areas = list(self.game_config['generated']['areas']['circles'].keys())
@@ -171,6 +172,57 @@ class ConfigManager:
             choices = sorted(choices, key=lambda choice: len(choice['name']), reverse=reverse)
         return choices
 
+    def getLeaders(self, faction):
+        selected = []
+        values = self.deck_generator['traitor_deck']['values']
+        for entry in values:
+            leader = entry[0]
+            leader_name = entry[1]
+            leader_faction = entry[2]
+            if leader_faction == faction:
+                selected.append(leader)
+        return selected
+
+    def getLeadersFaction(self, leader):
+        selected = []
+        values = self.deck_generator['traitor_deck']['values']
+        for entry in values:
+            leader_id = entry[0]
+            leader_faction = entry[2]
+            if leader_id == leader:
+                return leader_faction
+        return None
+
+    def getLeadersNamesStrength(self, lst):
+        selected = []
+        values = self.deck_generator['traitor_deck']['values']
+        for entry in values:
+            leader = entry[0]
+            if leader in lst:
+                selected.append(entry)
+        return selected
+
+    def getLeadersChoices(self, faction, sort=False, reverse=False, swap=False):
+        choices = []
+        values = self.deck_generator['traitor_deck']['values']
+        for entry in values:
+            leader = entry[0]
+            leader_name = entry[1]
+            leader_faction = entry[2]
+            if leader_faction != faction:
+                continue
+            name, value = leader, leader_name
+            if swap:
+                value, name = leader, leader_name
+            choices.append({
+                'name': name,
+                'value': value
+            })
+        if sort:
+            choices = sorted(choices, reverse=reverse)
+        return choices
+
+
     def getDeckChoices(self):
         return [
             {
@@ -422,20 +474,19 @@ class RenderingProcessor:
                 if wheel not in game_state['areas'].keys():
                     to_remove_points.append(area)
                 continue
-            for token_object in game_state['visual'][area]:
-                if type(token_object) is str:
-                    continue
-                region = token_object['region']
-                token = token_object['token']
-                removal = None
-                if area not in game_state['areas'].keys():
-                    removal = area, region, token
-                elif region not in game_state['areas'][area].keys():
-                    removal = area, region, token
-                elif token not in game_state['areas'][area][region].keys():
-                    removal = area, region, token
-                if removal is not None:
-                    to_remove.append(removal)
+            for token_region in game_state['visual'][area].keys():
+                for token in game_state['visual'][area][token_region].keys():
+                    region = token_region
+                    removal = None
+                    if area not in game_state['areas'].keys():
+                        removal = area, region, token
+                    elif region not in game_state['areas'][area].keys():
+                        removal = area, region, token
+                    elif token not in game_state['areas'][area][region].keys():
+                        removal = area, region, token
+                    print(removal)
+                    if removal is not None:
+                        to_remove.append(removal)
         # remove gathered
         for area in to_remove_points:
             del game_state['visual'][area]
@@ -556,11 +607,34 @@ class OriginatorTruthsayer(OriginatorJSON):
     def kill(self, leader):
         if not self.processor.manager.isLeader(leader):
             raise ValueError('Invalid leader')
-        self._object_state['areas']['tleilaxu_tanks'][leader] = 1
+        faction = self.processor.manager.getLeadersFaction(leader)
+        if leader not in self._object_state['hidden']['leaders'][faction]:
+            raise ValueError('Leader not in players hand')
+        self._object_state['hidden']['leaders'][faction].remove(leader)
+        if 'tleilaxu_tanks' not in self._object_state['areas'].keys():
+            self._object_state['areas']['tleilaxu_tanks'] = {
+                'whole': {}
+            }
+        self._object_state['areas']['tleilaxu_tanks']['whole'][leader] = 1
         cmd = '/{0} {1}'.format('kill', leader)
         self.appendCMD(cmd)
 
-    def revive(self, leader_or_unit, N=1):
+    def reviveLeader(self, leader):
+        if not self.processor.manager.isLeader(leader):
+            raise ValueError('Invalid leader')
+        if 'tleilaxu_tanks' not in self._object_state['areas'].keys():
+            raise ValueError('Leader not in Tleilaxu Tanks')
+        if 'whole' not in self._object_state['areas']['tleilaxu_tanks'].keys():
+            raise ValueError('Leader not in Tleilaxu Tanks')
+        if leader not in self._object_state['areas']['tleilaxu_tanks']['whole'].keys():
+            raise ValueError('Leader not in Tleilaxu Tanks')
+        faction = self.processor.manager.getLeadersFaction(leader)
+        del self._object_state['areas']['tleilaxu_tanks']['whole'][leader]
+        self._object_state['hidden']['leaders'][faction].append(leader)
+        cmd = '/{0} {1}'.format('reviveLeader', leader)
+        self.appendCMD(cmd)
+
+    def revive(self, faction, n, special=False):
         pass
 
     def lead(self, player, leader):
@@ -689,12 +763,15 @@ class OriginatorTruthsayer(OriginatorJSON):
         return game_id, card
 
     def hand(self, faction):
+        leaders_list = self._object_state['hidden']['leaders'][faction]
+        leaders_data = self.processor.manager.getLeadersNamesStrength(leaders_list)
         return {
             'game_id': self._object_state['meta']['texts']['game_id'],
             'faction_name': self.processor.game_config['faction_names'][faction],
             'cards': self._object_state['hidden']['cards'][faction],
             'spice': self._object_state['hidden']['spice'][faction],
-            'reserves': self._object_state['hidden']['reserves'][faction]
+            'reserves': self._object_state['hidden']['reserves'][faction],
+            'leaders': leaders_data
         }
 
     def join(self, seat, username, player_id, faction='random'):
@@ -776,11 +853,13 @@ class OriginatorTruthsayer(OriginatorJSON):
             'storm': deck_storm,
             'traitor': deck_traitor
         }
+        hidden['leaders'] = {}
         areas = {
             'storm': random.randint(1, 18)
         }
         for player, faction in meta['factions'].items():
             hidden['cards'][faction] = []
+            hidden['leaders'][faction] = self.processor.manager.getLeaders(faction)
             if faction == 'atreides':
                 hidden['reserves']['atreides'] = {
                     'atreides_troops': 10
@@ -880,7 +959,6 @@ class OriginatorTruthsayer(OriginatorJSON):
 
     def render(self, outfile, battle=False):
        self._object_state = self.processor.process(self._object_state)
-       self.backup()
        renderer = Renderer(
            self._object_state,
            self.processor.game_config,
